//
//  CroppingImageView.m
//  Cropped Image
//
//  Created by jcr on Tue Jul 16 2002.
//  Copyright (c) 2002 Apple Computer, Inc.  All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "MMSPhoto.h"
#import "CropMarker.h"

//@class CropMarker;

@interface CroppingImageView : NSImageView
{
    id selectionMarker;
    id prototypeMarker;
	BOOL shouldAntiAlias;
    NSMutableArray *selectionMarkers;
    BOOL isMarkerMove;
    BOOL imageTableDragBegun;
    BOOL isNewMarker;
    NSImage *offscreenImage;
    NSURL *droppedURL;
    MMSPhoto *currentPhoto;
    NSTrackingRectTag myTrackingRect;
    NSMutableSet *unspecifiedMarkers;
    id editController;
	
	NSArray *facesInfoDictArray;
	NSColor *backgroundColor;
}

- (IBAction) takeSelectionMarkerStyleFrom:sender;
- (IBAction) takeSelectionColorFrom:sender;
- (IBAction) takeAntiAliasModeFrom:sender;
- (IBAction) takeContinuousModeFrom:sender;
- (NSImage *) croppedImage;
- (void) selectionChanged;
- (void) setSelectionMarker:(CropMarker *) marker;
- (id)selectionMarker;

// Notifications
- (void)postSelectionChangedNotification;
- (void)postPeoplePickerNeedsCloseNotification;
- (void)postMetaDataChangedNotification;


// Drawing methods
- (void)configureUnspecifedMarkers;
- (void)drawAllMarkers;
- (void)deselectAllMarkers;
- (void)deleteSelectedMarkers;
- (void)createNewSelectionMarker;
- (void)createNewSelectionMarkerWithPerson:(MMSPerson *)aPerson;
- (void)hideAllMarkers;
- (void)showAllMarkers;
- (void)removeAllPersonLabelViews;
- (void)deleteAllLabelViews;
- (void)drawFaceMarkers;

// Accessors
- (NSColor *)backgroundColor;
- (void)setBackgroundColor:(NSColor *)aBackgroundColor;

- (id)editController;
- (void)setEditController:(id)anEditController;

- (NSArray *)facesInfoDictArray;
- (void)setFacesInfoDictArray:(NSArray *)aFacesInfoDictArray;

- (NSMutableArray *)selectionMarkers;
- (void)setSelectionMarkers:(NSMutableArray *)value;
- (BOOL)isMarkerMove;
- (void)setIsMarkerMove:(BOOL)flag;
- (void)setPrototypeMarker: marker;
- (id)prototypeMarker;
- (BOOL)imageTableDragBegun;
- (void)setImageTableDragBegun:(BOOL)flag;
- (NSURL *)droppedURL;
- (void)setDroppedURL:(NSURL *)aDroppedURL;
- (MMSPhoto *)currentPhoto;
- (void)setCurrentPhoto:(MMSPhoto *)aCurrentPhoto;
- (BOOL)isNewMarker;
- (void)setIsNewMarker:(BOOL)flag;
- (NSMutableSet *)unspecifiedMarkers;
- (void)setUnspecifiedMarkers:(NSMutableSet *)aSet;


// Mouse handling
- (CropMarker *)markerUnderPoint:(NSPoint)aPoint;
//- (void)selectAndTrackMouseWithEvent:(NSEvent *)theEvent;
- (void)trackKnob:(NSInteger)knob 
         ofMarker:(CropMarker *)marker 
        withEvent:(NSEvent *)theEvent;

// Serialization Support
- (NSArray *)serializedSelectionMarkers;
- (void)writeCroppedImagesToBasePath:(NSString *)aBasePath;

// Offsceen
- (NSImage *)offscreenImage;
- (void)setOffscreenImage:(NSImage *)anOffscreenImage;
- (NSImage *)_offscreenImage;
- (void)createOffscreenImage;
- (CGFloat)photoReduction;
- (NSRect)photoRectInImageView;
- (CGFloat)imageBorderWidth;
- (CGFloat)imageBorderHeight;

// Convenience
- (NSNumber *)imagePixelsWide;
- (NSNumber *)imagePixelsHigh;

- (void)resetTrackingRect;


@end

@interface NSImageCell (CroppingImageView)

- (NSRect) rectCoveredByImageInBounds:(NSRect) bounds;

@end

#define selectionChangedNotification @"ImageSelectionChanged"
#define metaDataChangedNotification @"MetaDataChanged"
#define imageLoadedNotification @"ImageLoaded"
#define personLabelNeededNotification @"PersonLabelNeeded"
#define urlDroppedNotification @"URLDroppedNotification"
#define peoplePickerNeedsCloseNotification @"PeoplePickerNeedsCloseNotification"

