//
//  MMSLocation.h
//  MemoryMiner
//
//  Created by John Fox on 10/29/04.
//  Copyright 2004 MemoryMiner Software. All rights reserved.
//

#import "MMSPersistentObject.h"

#import <Foundation/Foundation.h>
#import <AddressBook/AddressBook.h>

@interface MMSLocation : MMSPersistentObject {
    NSString *address;
    NSString *name;
    NSString *city;
    NSString *state;
    NSString *postalCode;
    NSString *country;
    NSString *longitude;
    NSString *latitude;
    NSString *altitude;
    NSString *mapServerURL;
    NSString *countryISOCode;
    
    BOOL shouldPerformCityOnlyGeocodeLookup;
    BOOL cityOnlyGeocodeLookupInProgress;
    BOOL geocodeLookupInProgress;
}

+ (NSArray *)locationsMatchingSearchWords:(NSArray *)aWordArray;
+ (MMSLocation *)newOrExistingLocationWithName:(NSString *)aName;
+ (MMSLocation *)newWithName:(NSString *)aName;
+ (void)createNewLocationsFromABRecord:(ABRecord *)anABRecord;
+ (NSArray *)allLocationsSortedByName;
+ (NSImage *)defaultMapImage;

+ (NSDictionary *)msnMapsCountryCodesDict;
+ (NSDictionary *)isoCountryCodesDict;
+ (NSString *)mapsFolderName;

//
// Conveniences
//
- (NSString *)addressStringForLookup;
- (NSString *)locationStringForGoogleLookupIncludingAddress:(BOOL)includeAddress;
- (NSURL *)mapquestURL;
- (NSURL *)mapquestCoordinateURL;
- (NSURL *)msnMapsURL;
- (NSURL *)msnMapsURLIncludingAddress:(BOOL)includeAddresss;
- (NSString *)isoCountryCode;
- (NSString *)msnMapsCountryCode;
- (NSURL *)googleMapsURL;
- (NSURL *)googleSatelliteMapsURL;
- (NSString *)multiLineDisplayString;
- (NSString *)singleLineDisplayString;
- (NSString *)formattedDisplayStringUsingReturns:(BOOL)useReturns;
- (NSString *)twoLineNameAndAddress;

- (NSImage *)iconImage;
- (NSArray *)photosSortedByDate;
- (NSArray *)activePhotosSortedByDate;
- (NSArray *)photosSortedByDateShowingActiveOnly:(BOOL)showActiveOnly;
- (NSImage *)defaultMapImage;
- (NSImage *)mapFileImage;
- (NSURL *)mapFileURL;
- (NSString *)mapFilePath;
- (NSImage *)overviewMapImage;
- (NSImage *)mapImageAtIndex:(NSInteger)anIndex;

- (NSString *)mapFileBasePath;
- (NSArray *)mapFileImageArray;
- (NSArray *)mapFilePathArray;
- (NSArray *)mapFileFullPathArray;
- (BOOL)createMapFileBasePath;
- (BOOL)exportMapFilesToPath:(NSString *)aBasePath;

- (BOOL)hasAssignedPhotos;
- (BOOL)hasLatLongValues;
- (BOOL)hasCustomMapFile;
- (NSInteger)assignedPhotoCount;

- (NSMutableDictionary *)latLongValueDict;

//
// Action methods
//
- (void)performGeoCodeLookup;

//
// Flickr support
//

- (NSArray *)flickrGeoArgsWithPhotoID:(NSString *)aPhotoID;

// 
// Accessors
//
- (NSString *)address;
- (void)setAddress:(NSString *)anAddress;

- (NSString *)name;
- (void)setName:(NSString *)aName;

- (NSString *)city;
- (void)setCity:(NSString *)aCity;

- (NSString *)countryISOCode;
- (void)setCountryISOCode:(NSString *)anIsoCode;

- (NSString *)state;
- (void)setState:(NSString *)aState;

- (NSString *)postalCode;
- (void)setPostalCode:(NSString *)aPostalCode;

- (NSString *)country;
- (void)setCountry:(NSString *)aCountry;

- (NSString *)longitude;
- (void)setLongitude:(NSString *)aLongitude;

- (NSString *)latitude;
- (void)setLatitude:(NSString *)aLatitude;

- (NSString *)altitude;
- (void)setAltitude:(NSString *)anAltitude;

- (NSString *)mapServerURL;
- (void)setMapServerURL:(NSString *)aURL;

- (BOOL)geocodeLookupInProgress;

@end

#define newLocationCreatedNotification @"MMSLocationCreatedNotification"

