//
//  MMSSelectionMarker.h
//  MemoryMiner
//
//  Created by John Fox on 10/6/04.
//  Copyright 2004 MemoryMiner Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import "CropMarker.h"
#import "MMSPerson.h"
#import "MMSPhoto.h"
#import "DataCrux.h"

@interface MMSSelectionMarker : CropMarker 
{
    NSNumber *originX;
    NSNumber *originY;
    NSNumber *width;
    NSNumber *height;
    NSNumber *scalingFactor;
    NSBox *borderBox;
    NSString *thumbnailURL;
    NSString *notes;
    MMSPhoto *photo;
	NSMutableDictionary *XMPAnnotationDict;
	
	NSSize photoSize;
	
	BOOL isMouseInMarker;
	
	// Undo support
	NSValue *editedSelectedRectValue;
}

@property(retain) NSValue *editedSelectedRectValue;

//
// Creation
//
+ (id)newMarkerForPhoto:(MMSPhoto *)aPhoto
             withPerson:(MMSPerson *)aPerson;

// Persistence
- (void)storeUsingDataCrux:(DataCrux *)aDataCrux;
- (void)congfigurePersistenRectValues;
+ (void)saveMarkers:(NSArray *)aSelectionMarkerArray;
- (NSURL *)thumbnailStorageURL;
- (NSString *)derivedThumbnailPath;
- (NSString *)photoNameBasedThumbnailPath;
- (NSString *)croppedImageThumbnailPath;

// Conveniences
+ (NSRect)scaledFocusRectFromRect:(NSRect)aRect
                         forPhoto:(MMSPhoto *)aPhoto
                    scaledForSize:(NSSize)aSize;

+ (NSRect)scaledFocusRectFromRect:(NSRect)aRect
				 forPhotoWithSize:(NSSize)aPhotoSize
                    scaledForSize:(NSSize)aTargetSize;

- (void)saveCroppedJPEG;
- (void)createLifePeriodIconIfNeeded;
- (void)deleteThumbnailFile;
- (NSImage *)imageFromStoredThumbnail;
- (NSImage *)iconImage;
- (NSImage *)fullSizedImage;
- (NSImage *)fullResImage;
- (NSRect)fullSizeRect;
- (NSRect)selectionRectScaledForImageSize:(NSSize)aSize;
- (void)configureSelectectedRect;
- (void)clampDimensionsIfNeeded;
- (NSImage *)personIconSourceImage;
- (NSString *)personDisplayAge;
- (NSString *)personNameAndAge;
- (NSView *)personLabelView;

//
// Geometry
//

- (void)configureForView:(NSView *)aView
		   withPhotoSize:(NSSize)aPhotoSize;

- (CGFloat)upperLeftXRatio;
- (CGFloat)upperLeftYRatio;
- (CGFloat)widthRatio;
- (CGFloat)heightRatio;

- (void)setRatioValuesWithDBUpdate:(BOOL)aChoice;

- (void)configureSizeForUnspeciedMarker;
- (void)configureUnspecifiedGeometry;

- (BOOL)hasPostiveXOrigin;

//
// XML Serialization
//
- (NSString *)personName;
- (NSString *)personNameAndCaption;
- (NSString *)photoObjectID;
- (NSString *)photoUUID;
- (NSString *)personObjectID;
- (NSString *)relativeThumbnailPath;
- (NSString *)relativePersonIconPath;
- (NSNumber *)exportWidth;
- (NSNumber *)exportHeight;
- (NSNumber *)exportOriginX;
- (NSNumber *)exportOriginY;
- (NSNumber *)x1;
- (NSNumber *)y1;
- (NSNumber *)x2;
- (NSNumber *)y2;

//
// Undo Support
//

- (void)setEditedSelectedRectValue:(NSValue *)aRectValue;

//
// Flickr
//
- (NSArray *)flickrNotesArgsWithPhotoID:(NSString *)aPhotoID;

//
// MMSCoverflowImage protocol
//
- (NSString *)coverflowImagePath;
- (NSString *)lowRezCoverflowImagePath;

// Accessors
- (NSString *)notes;
- (void)setNotes:(NSString *)value;

- (MMSPhoto *)photo;
- (void)setPhoto:(MMSPhoto *)aPhoto;

- (NSSize)photoSize;
- (void)setPhotoSize:(NSSize)aPhotoSize;

- (NSNumber *)originX;
- (void)setOriginX:(NSNumber *)anOriginX;

- (NSNumber *)originY;
- (void)setOriginY:(NSNumber *)anOriginY;

- (NSNumber *)width;
- (void)setWidth:(NSNumber *)aWidth;

- (NSNumber *)height;
- (void)setHeight:(NSNumber *)aHeight;

- (void)setIsMouseInMarker:(BOOL)aChoice;
- (BOOL)isMouseInMarker;

- (NSNumber *)scalingFactor;
- (void)setScalingFactor:(NSNumber *)aScalingFactor;

- (NSString *)thumbnailURL;
- (void)setThumbnailURL:(NSString *)aThumbnailURL;

- (NSBox *)borderBox;
- (void)setBorderBox:(NSBox *)aBorderBox;

- (NSMutableDictionary *)XMPAnnotationDict;
- (void)setXMPAnnotationDict:(NSMutableDictionary *)anXMPAnnotationDict;

@end

#define selectionMarkerChosenNotification @"MMSSelectionMarkerChosenNotification"

