//
//  MMSWebServiceHelper.h
//  MMSBase
//
//  Created by John Fox on 12/1/06.
//  Copyright 2006 GroupSmarts, LLC. All rights reserved.
//

#import "MMSRESTInvocation.h"

//#import "MMSPersistentObject.h"

@interface MMSWebServiceHelper : NSObject 
{
    NSMutableURLRequest *currentRequest;
    NSURLConnection *currentConnection;
    NSURLResponse *currentResponse;
    NSMutableData *receivedData;
    NSString *authString;
}

+ (id)sharedHelper;

//
// Conveniences
//

- (NSString *)currentResponseDataAsString;

- (NSURL *)objectResourceURLWithUUID:(NSString *)aUUID
                             andType:(NSString *)anObjectType
                         isDeepFetch:(BOOL)isDeepFetch;

- (BOOL)remoteObjectExistsForLocalObject:(id)anObject;

- (NSString *)wsURLBase;

//
// Message creation
//

- (void)createAndSendMessageWithMethodName:(NSString *)aMethodName
                          parameterObjects:(NSArray *)someParemeterObjects
                               endPointURL:(NSURL *)anEndPointURL;

- (NSMutableString *)createXMLRPCMessageForMethodNamed:(NSString *)aMethodName
                                      parameterObjects:(NSArray *)aParameterObjectArray;

- (NSString *)createWSUserAccountFromDict:(NSDictionary *)aUserDict;

- (void)createAndSendRESTMessageOfType:(NSString *)anHTTPMethod
                         toEndPointURL:(NSURL *)anEndPointURL
                         usingInfoDict:(NSDictionary *)anInfoDict;

- (void)createAndSendRESTMessageOfType:(NSString *)anHTTPMethod
                         toEndPointURL:(NSURL *)anEndPointURL
                         usingInfoDict:(NSDictionary *)anInfoDict
                   secondaryInvocation:(MMSRESTInvocation *)aSecondaryInvocation;

- (MMSRESTInvocation *)createRESTMessageOfType:(NSString *)anHTTPMethod
                                 toEndPointURL:(NSURL *)anEndPointURL
                                 usingInfoDict:(NSDictionary *)anInfoDict;

- (NSString *)xmlRPCStructForObject:(id)anObject
                          usingKeys:(NSArray *)aKeyArray
                        indentCount:(NSInteger)anIndentCount;

- (NSDictionary *)portfolioCreationDictFromInfoDict:(NSDictionary *)anInfoDict
                                          andPhotos:(NSArray *)aPhotoArray;

//
// Remote Object Creation
//

- (NSURL *)objectResourceURLWithUUID:(NSString *)aUUID
                             andType:(NSString *)anObjectType;

- (void)recordPeopleInArray:(NSArray *)aPersonArray
           usingEndPointURL:(NSURL *)anEndPointURL;

- (void)recordSelectionMarkersInArray:(NSArray *)aSelectionMarkerArray;
- (void)recordSelectionMarkersInArray:(NSArray *)aSelectionMarkerArray
                       usingWSURLBase:(NSString *)aWSURLBase;

- (void)recordLocationsInArray:(NSArray *)aLocationArray;

- (void)recordLocationsInArray:(NSArray *)aLocationArray
              usingEndPointURL:(NSURL *)anEndPointURL;

- (void)recordPeopleInArray:(NSArray *)aPersonArray;

- (void)createCollectionFromInfoDict:(NSDictionary *)anInfoDict
                          withPhotos:(NSArray *)aPhotoArray;

- (void)createMediaObjectsUsingInfoDictArray:(NSArray *)anInfoDictArray;
- (NSArray *)createMediaObjectsUsingInfoDictArray:(NSArray *)anInfoDictArray
                                         delegate:(id)aDelegate;

- (void)createPlacementsForPhotos:(NSArray *)aPhotoArray;

- (void)createPeopleForPhotos:(NSArray *)aPhotoArray;

- (void)createLocationsForPhotos:(NSArray *)aPhotoArray;

//
// Remote Object Deletion
//
- (void)deleteObjectOfType:(NSString *)anObjectType
                  withUUID:(NSString *)aUUID;

//
// Accessors
//

- (NSString *)authString;
- (void)setAuthString:(NSString *)anAuthString;

- (NSURLConnection *)currentConnection;
- (void)setCurrentConnection:(NSURLConnection *)aCurrentConnection;

- (NSMutableURLRequest *)currentRequest;
- (void)setCurrentRequest:(NSMutableURLRequest *)aCurrentRequest;

- (NSURLResponse *)currentResponse;
- (void)setCurrentResponse:(NSURLResponse *)aCurrentResponse;


@end
