//
//  SQLDatabase.h
//  An objective-c wrapper for the SQLite library
//  available at http://www.hwaci.com/sw/sqlite/
//
//  Created by Dustin Mierau on Tue Apr 02 2002.
//  Copyright (c) 2002 Blackhole Media, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

#if DC_SQLITE_VERSION==3
    #import "sqlite3.h"
#else
    #import "sqlite.h"
#endif


@class DC_SQLResult;
@class DC_SQLRow;

@interface DC_SQLDatabase : NSObject 
{

#if DC_SQLITE_VERSION == 3
	sqlite3*	mDatabase;
#else
	sqlite*		mDatabase;    
#endif
	NSString*	mPath;
}

+ (id)databaseWithFile:(NSString*)inPath;
- (id)initWithFile:(NSString*)inPath;
- (BOOL)open;
- (void)close;

+ (NSString*)prepareStringForQuery:(NSString*)inString;
- (DC_SQLResult*)performQuery:(NSString*)inQuery;
- (DC_SQLResult*)performQueryWithFormat:(NSString*)inFormat, ...;

@end

@interface DC_SQLResult : NSObject
{
	char**	mTable;
	NSInteger		mRows;
	NSInteger		mColumns;
        NSInteger             last_id;
}

- (NSInteger)rowCount;
- (NSInteger)columnCount;
- (NSInteger)lastID;

// !!!:scott:20050516 

- (char **)rawSqliteData;


- (DC_SQLRow*)rowAtIndex:(NSInteger)inIndex;
- (NSEnumerator*)rowEnumerator;

@end

@interface DC_SQLRow : NSObject
{
	char**	mRowData;
	char**	mColumns;
	NSInteger		mColumnCount;
}

- (NSInteger)columnCount;

- (NSString*)nameOfColumnAtIndex:(NSInteger)inIndex;
- (NSString*)nameOfColumnAtIndexNoCopy:(NSInteger)inIndex;

- (NSString*)stringForColumn:(NSString*)inColumnName;
- (NSString*)stringForColumnNoCopy:(NSString*)inColumnName;
- (NSString*)stringForColumnAtIndex:(NSInteger)inIndex;
- (NSString*)stringForColumnAtIndexNoCopy:(NSInteger)inIndex;

@end


@interface SQLQueryBuilder : NSObject {

    NSString *myTable;
}

- (id)initWithTable:(NSString *)tableName;

+ (NSString *)stringForInsertIntoTable:(NSString *)tableName withColumnValuesDictionary:(NSDictionary *)fieldValues;
+ (NSString *)stringForUpdateOfTable:(NSString *)tableName withColumnValuesDictionary:(NSDictionary *)fieldValues constraints:(NSDictionary *)mask;
+ (NSString *)stringForDeleteFromTable:(NSString *)tableName constraints:(NSDictionary *)mask;
+ (NSString *)stringForSelectFromTable:(NSString *)tableName fields:(NSArray *)fields constraints:(NSDictionary *)mask;
+ (NSString *)stringForSelectFromTables:(NSArray *)tableName fields:(NSArray *)fields constraints:(NSDictionary *)mask;
+ (NSString *)stringForCreateTable: (NSString *)tableName withFields: (NSArray *)fields andPrimaryKey: (NSString *)primaryKey;
+ (NSString *)stringForCreateTable: (NSString *)tableName withFields: (NSArray *)fields andPrimaryKey: (NSString *)primaryKey isTemporary:(BOOL)isTemp;
+ (NSString *)stringForNextAutoincrementValueInTable: (NSString *)tableName withPrimaryKey:(NSString *)primaryKey;


@end