@import <AppKit/CPView.j>
@import "MKMapItem.j"
@import "MKMapView.j"
@import "MKLocation.j"

@implementation MKMarker : MKMapItem
{
    Marker      _gMarker    @accessors(property=gMarker);
    MKLocation  _location   @accessors(property=location);
    BOOL        _draggable  @accessors(property=draggable);
    BOOL        _withShadow  @accessors(property=withShadow);
    CPString    _iconUrl    @accessors(property=iconUrl);
    CPString    _shadowUrl  @accessors(property=shadowUrl);
	MKMapView	_mapView	@accessors(property=mapView);
    id          _delegate  @accessors(property=delegate);
}

+ (MKMarker)marker
{
    return [[MKMarker alloc] init];
}

- (id)initAtLocation:(MKLocation)aLocation
{
    if (self = [super init]) {
        _location = aLocation;
		_withShadow = YES;
		_draggable = YES;
		_gMarker = nil;
    }
	
    return self;
}

- (void)init
{
	self = [super init];
	
	
	return self;
}

- (void)didClick
{	
	if (_delegate && [_delegate respondsToSelector:@selector(didClickOnMarker:)]) 
	{
        [_delegate didClickOnMarker:self];
	}
}

- (void)didMouseover
{	
	//log('MKMarker in didMouseover');
	
	if (_delegate && [_delegate respondsToSelector:@selector(didMousoverMarker:)]) 
	{
        [_delegate didMousoverMarker:self];
	}
}

- (void)didMouseout
{	
	//log('MKMarker in didMouseout');

	if (_delegate && [_delegate respondsToSelector:@selector(didMouseoutMarker:)]) 
	{
        [_delegate didMouseoutMarker:self];
	}
}

- (void)updateLocation
{
    _location = [[MKLocation alloc] initWithLatLng:_gMarker.getLatLng()];
    	
    if (_delegate && [_delegate respondsToSelector:@selector(mapMarker:didMoveToLocation:)]) {
        [_delegate mapMarker:self didMoveToLocation:_location];
    }
}

/*!
    Sets the icon URL based on this url pattern:
    http://maps.google.com/mapfiles/ms/micons/<anIconName>.png
    
    Some examples:
    
    POI
    arts
    bar
    blue-dot
    blue-pushpin
    blue
    bus
    cabs
    camera
    campfire
    campground
    caution
    coffeehouse
    convienancestore
    cycling
    dollar
    drinking_water
    earthquake
    electronics
    euro
    fallingrocks
    ferry
    firedept
    fishing
    flag
    gas
    golfer
    green-dot
    green
    grn-pushpin
    grocerystore
    groecerystore
    helicopter
    hiker
    homegardenbusiness
    horsebackriding
    hospitals
    hotsprings
    info
    info_circle
    landmarks-jp
    lightblue
    lodging
    ltblu-pushpin
    ltblue-dot
    man
    marina
    mechanic
    motorcycling
    movies
    orange-dot
    orange
    parkinglot
    partly_cloudy
    pharmacy-us
    phone
    picnic
    pink-dot
    pink-pushpin
    pink
    plane
    police
    postoffice-jp
    postoffice-us
    purple-dot
    purple-pushpin
    purple
    question
    rail
    rainy
    rangerstation
    realestate
    recycle
    red-dot
    red-pushpin
    red
    restaurant
    sailing
    salon
    shopping
    ski
    ski
    snack_bar
    snowflake_simple
    sportvenue
    subway
    sunny
    swimming
    toilets
    trail
    tram
    tree
    truck
    volcano
    water
    waterfalls
    webcam
    wheel_chair_accessible
    woman
    yellow-dot
    yellow
    yen
    ylw-pushpin
    
    You can find a list of official google maps icons here:
    http://www.visual-case.it/cgi-bin/vc/GMapsIcons.pl
*/
- (void)setGoogleIcon:(CPString)anIconName withShadow:(BOOL)withShadow
{
    _withShadow = withShadow;
    
    if (anIconName) {
        _iconUrl = "http://maps.google.com/mapfiles/ms/micons/" + anIconName + ".png"
        
        if (withShadow) {
            if (anIconName.match(/dot/) || anIconName.match(/(blue|green|lightblue|orange|pink|purple|red|yellow)$/)) {
                _shadowUrl = "http://maps.google.com/mapfiles/ms/micons/msmarker.shadow.png";
            } else if (anIconName.match(/pushpin/)) {
                _shadowUrl = "http://maps.google.com/mapfiles/ms/micons/pushpin_shadow.png";
            } else {
                _shadowUrl = "http://maps.google.com/mapfiles/ms/micons/" + anIconName + ".shadow.png";
            }
        }
    } else {
        _iconUrl = nil;
        _shadowUrl = "http://maps.google.com/mapfiles/ms/micons/msmarker.shadow.png"; //default shadow
    }
}

- (void)enableDragging:(BOOL)aChoice
{
	if (_gMarker)
	{
		if (aChoice)
		{
			_gMarker.enableDragging();
		}
		else
		{
			_gMarker.disableDragging();
		}
	}
}
- (void)setGoogleIcon:(CPString)anIconName
{
    [self setGoogleIcon:anIconName withShadow:YES];
}


- (void)addToMapView:(MKMapView)aMapView
{
    var googleMap = [aMapView gMap];
    var gm = [MKMapView gmNamespace];
    
    var icon = new gm.Icon(gm.DEFAULT_ICON);
    
    // set a different icon if the _iconUrl is set
    if (_iconUrl) {
        icon.image = _iconUrl;
        icon.iconSize = new gm.Size(32, 32);
        icon.iconAnchor = new gm.Point(16, 32);
    }
    
    // set the shadow
    if (_withShadow && _shadowUrl) {
        icon.shadow = _shadowUrl;
        icon.shadowSize = new gm.Size(59, 32);
    }
    
    var markerOptions = { "icon":icon, "clickable":false, "draggable":_draggable };    
    _gMarker = new gm.Marker([_location googleLatLng], markerOptions);

    gm.Event.addListener(_gMarker, 'dragend', function() { [self updateLocation]; });
    gm.Event.addListener(_gMarker, 'click', function() { [self didClick]; });
    gm.Event.addListener(_gMarker, 'mouseover', function() { [self didMouseover]; });
    gm.Event.addListener(_gMarker, 'mouseout', function() { [self didMouseout]; });
    googleMap.addOverlay(_gMarker);
		
	[[aMapView markerArray] addObject:self];
	
	_mapView = aMapView;
}

- (void)removeFromMapView:(MKMapView)aMapView
{
    [[aMapView markerArray] removeObject:self];

    var gm = [aMapView gMap];

    gm.removeOverlay([self gMarker]);
}

- (void)encodeWithCoder:(CPCoder)coder
{
    [coder encodeObject:[[_location latitude], [_location longitude]] forKey:@"location"];
}

- (void)setHidden:(BOOL)isHidden
{
	if (![self gMarker])
	{
		return;
	}
	
	if (isHidden)
	{
		[self gMarker].hide();
	}
	else
	{
		[self gMarker].show();
	}
}



@end

